* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm="Scale" THEN DELETE; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP StdErr Gradient Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6b          *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example6b;

* Import trimmed example data excel data file from sheet "skin cancer" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example6b_Data.xlsx" OUT=work.Example6b DBMS=XLSX REPLACE; 
     SHEET="skin cancer"; GETNAMES=YES; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example6b_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

* Data preparation;
DATA work.Example6b; SET work.Example6b; 
* Add labels to original variables;
  LABEL nation=    "nation: String-Format Nation" 
        regionID=  "regionID: L2 Region ID Variable"
        countyID=  "countyID: L1 County ID Variable"
        deaths=    "deaths: Count Outcome of Actual Deaths"
        expected=  "expected: Offset for Expected Deaths"
        uv=        "uv: County Amount of UV Exposure";
* Filter to complete cases before computing cluster means;
  IF CMISS(regionID,deaths,uv,nation)>0 THEN DELETE;
RUN;

* Create cluster=region means;
PROC SORT DATA=work.Example6b; BY regionID; RUN;
PROC MEANS NOPRINT N DATA=work.Example6b; 
     BY regionID; VAR deaths uv;
     OUTPUT OUT=work.RegionMeans MEAN(deaths uv)= RMdeaths RMuv; RUN;
* Get sample size per region and label new region mean variables;
DATA work.RegionMeans; SET work.RegionMeans;
     regionN = _FREQ_; DROP _TYPE_ _FREQ_; 
     LABEL regionN=   "regionN: # Counties Contributing Data"
           RMdeaths=  "RMdeaths: Region Mean of Death Count"
           RMuv=      "RMuv: Region Mean of UV Exposure";
RUN;

TITLE "SAS Descriptive Statistics within Region-Level-2 Data";
PROC MEANS NOLABELS NDEC=2 DATA=work.RegionMeans; VAR regionN RMdeaths RMuv; RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.RegionMeans; VAR regionN RMdeaths RMuv; RUN; TITLE;

* Merge regionID means back with individual data;
DATA work.Example6b; MERGE work.Example6b work.RegionMeans; BY regionID;
  * Cluster-MC to get within-region deaths and UV;
    WRdeaths = deaths - RMdeaths;
    WRuv = uv - RMuv;
    LABEL WRdeaths=  "WRdeaths: Within-Region Deaths (0=RM)"
          WRuv=      "WRuv: Within-regionID UV Exposure (0=RM)";
RUN;

TITLE "SAS Descriptive Statistics within County-Level-1 Data";
PROC MEANS NOLABELS NDEC=3 N MEAN VAR STDDEV MIN MAX DATA=work.Example6b; 
     VAR deaths WRdeaths uv WRuv; RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.Example6b; VAR deaths WRdeaths uv WRuv; RUN; TITLE;
PROC FREQ DATA=work.Example6b; TABLE nation; RUN; TITLE;
PROC UNIVARIATE NOPRINT DATA=work.Example6b; VAR deaths;
      HISTOGRAM deaths / MIDPOINTS=0 TO 320 BY 20; 
RUN; QUIT; TITLE;


***********************************************************************************;
*******                    BEGIN EMPTY MODELS FOR EXAMPLE 6b                *******;
***********************************************************************************;

* To add an offset, first log-transform it within GLIMMIX: logexp=log(expected);
*    Then add OFFSET=logexp after / on MODEL line;
* I am using laplace estimation to try to match R results from glmer.nb;
* To use quadrature instead, swtich to METHOD=QUAD(QPOINTS=7) (7=STATA default);

TITLE "SAS Empty Means, Random Intercept Model for UV Exposure (predictor)";
PROC MIXED DATA=work.Example6b NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS regionID; 
     MODEL uv =  / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR=2 TYPE=UN SUBJECT=regionID; * VCORR gives ICC;
RUN; TITLE;

TITLE1 "SAS Model 1a: Empty Means, Single-Level for Deaths";
TITLE2 "Log Link, Poisson Conditional Distribution";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID; 
     MODEL deaths =  / SOLUTION LINK=LOG DIST=POISSON DDFM=NONE;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK is inverse link (to un-log);          
RUN; TITLE1; TITLE2;

TITLE1 "SAS Model 1b: Empty Means, Random Intercept for Deaths";
TITLE2 "Log Link, Poisson Conditional Distribution";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID;
     MODEL deaths =  / SOLUTION LINK=LOG DIST=POISSON DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=regionID;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK is inverse link (to un-log);
     COVTEST "Random Intercept?" 0;            * Test if G matrix (1,1)=0;
RUN; TITLE1; TITLE2;

TITLE1 "SAS Model 1c: Empty Means, Random Intercept for Deaths";
TITLE2 "Log Link, Negative Binomial Conditional Distribution";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID;
     MODEL deaths =  / SOLUTION LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=regionID;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK is inverse link (to un-log);
     ODS OUTPUT CovParms=CovEmpty1c;           * Save random int var for pseudo-R2;
RUN; TITLE1; TITLE2;


***********************************************************************************;
*******                BEGIN CONDITIONAL MODELS FOR EXAMPLE 6b              *******;
***********************************************************************************;

TITLE1 "SAS Model 2a: Add Fixed Slope of Region Mean UV";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID;
     MODEL deaths = RMuv / SOLUTION LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=regionID;
     ODS OUTPUT CovParms=CovBRuv2a;   * Save random int var for pseudo-R2;
RUN; TITLE1; 
* Calculate PseudoR2 relative to empty means model;
%PseudoR2(NCov=2, CovFewer=CovEmpty1c, CovMore1=CovBRuv2a);

TITLE1 "SAS Model 2b: Add Fixed Slope of Within-Region UV";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID;
     MODEL deaths = RMuv WRuv / SOLUTION LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=regionID;
* Request risk ratios for fixed slopes;
	ESTIMATE "UV Between Slope"     RMuv 1 / EXP;
	ESTIMATE "UV Within Slope"      WRuv 1 / EXP;
     ESTIMATE "UV Contextual Slope"  RMuv 1 WRuv -1 / EXP;
     ODS OUTPUT CovParms=CovWRuv2b;  * Save random int var for pseudo-R2;
RUN; TITLE1; 

TITLE1 "SAS Model 2c: Add Random Slope of Within-Region UV Predictor";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID;
     MODEL deaths = RMuv WRuv / SOLUTION LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT WRuv / GCORR TYPE=UN SUBJECT=regionID;
     COVTEST "Random WRuv Slope?"    . 0 0 .; * Leave (1,1) and OD, test(2,1)+(2,2)=0;
     ODS OUTPUT CovParms=CovRandWRuv2c;       * Save random variances for pseudo-R2;
RUN; TITLE1; 

TITLE1 "SAS Model 3a: Control for Main Effect of Nation on Intercept";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID nation;
     MODEL deaths =  RMuv WRuv nation / SOLUTION CHISQ LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT WRuv / GCORR TYPE=UN SUBJECT=regionID;
	LSMEANS nation / ILINK;           * Predicted unit-specific log count by nation;
     ODS OUTPUT CovParms=CovNatMain3a; * Save random variances for pseudo-R2;
RUN; TITLE1; 
* Calculate PseudoR2 relative to model 2c;
%PseudoR2(NCov=4, CovFewer=CovRandWRuv2c, CovMore1=CovNatMain3a);

TITLE1 "SAS Model 3b: Add Nation Interactions with UV Slopes";
PROC GLIMMIX DATA=work.Example6b NOCLPRINT NAMELEN=100 METHOD=LAPLACE GRADIENT;
     CLASS regionID nation;
     MODEL deaths = RMuv WRuv nation nation*RMuv nation*WRuv 
                     / SOLUTION CHISQ LINK=LOG DIST=NEGBIN DDFM=NONE;
     RANDOM INTERCEPT WRuv / GCORR TYPE=UN SUBJECT=regionID;
     ODS OUTPUT CovParms=CovNatUV3b;   * Save random variances for pseudo-R2;
	* Simple slopes per country for between and within UV effects;
     ESTIMATE "Between-Region UV: Belgium"      RMuv 1 RMuv*nation 1 0 0 0 0 0 0 0; 
     ESTIMATE "Between-Region UV: Denmark"      RMuv 1 RMuv*nation 0 1 0 0 0 0 0 0; 
     ESTIMATE "Between-Region UV: France"       RMuv 1 RMuv*nation 0 0 1 0 0 0 0 0; 
     ESTIMATE "Between-Region UV: Ireland"      RMuv 1 RMuv*nation 0 0 0 1 0 0 0 0; 
     ESTIMATE "Between-Region UV: Italy"        RMuv 1 RMuv*nation 0 0 0 0 1 0 0 0; 
     ESTIMATE "Between-Region UV: Netherlands"  RMuv 1 RMuv*nation 0 0 0 0 0 1 0 0; 
     ESTIMATE "Between-Region UV: UK"           RMuv 1 RMuv*nation 0 0 0 0 0 0 1 0; 
     ESTIMATE "Between-Region UV: W Germany"    RMuv 1 RMuv*nation 0 0 0 0 0 0 0 1; 
     ESTIMATE "Within-Region  UV: Belgium"       WRuv 1 WRuv*nation 1 0 0 0 0 0 0 0; 
     ESTIMATE "Within-Region  UV: Denmark"       WRuv 1 WRuv*nation 0 1 0 0 0 0 0 0; 
     ESTIMATE "Within-Region  UV: France"        WRuv 1 WRuv*nation 0 0 1 0 0 0 0 0; 
     ESTIMATE "Within-Region  UV: Ireland"       WRuv 1 WRuv*nation 0 0 0 1 0 0 0 0; 
     ESTIMATE "Within-Region  UV: Italy"         WRuv 1 WRuv*nation 0 0 0 0 1 0 0 0; 
     ESTIMATE "Within-Region  UV: Netherlands"   WRuv 1 WRuv*nation 0 0 0 0 0 1 0 0; 
     ESTIMATE "Within-Region  UV: UK"            WRuv 1 WRuv*nation 0 0 0 0 0 0 1 0; 
     ESTIMATE "Within-Region  UV: W Germany"     WRuv 1 WRuv*nation 0 0 0 0 0 0 0 1; 
RUN; TITLE1;
* Calculate PseudoR2 relative to model 2c;
%PseudoR2(NCov=4, CovFewer=CovRandWRuv2c, CovMore1=CovNatMain3a, CovMore2=CovNatUV3b);


* Close external file of saved results;
ODS RTF CLOSE;


